@@ -21,11 +21,14 @@
 #include "bson.h"
 #include "mongoc.h"
 
+#include "phongo_compat.h"
+#include "php_phongo_classes.h"
+
 #define phpext_mongodb_ptr &mongodb_module_entry
 extern zend_module_entry mongodb_module_entry;
 
 /* FIXME: Its annoying to bump version. Move into phongo_version.h.in */
-#define PHP_MONGODB_VERSION "1.3.3"
+#define PHP_MONGODB_VERSION "1.4.0"
 #define PHP_MONGODB_STABILITY "stable"
 
 /* Structure for persisted libmongoc clients. The PID is included to ensure that
@@ -61,8 +64,6 @@ ZEND_TSRMLS_CACHE_EXTERN()
 
 #define PHONGO_WRITE_CONCERN_W_MAJORITY "majority"
 
-#include "php_phongo_classes.h"
-
 /* This enum is necessary since mongoc_server_description_type_t is private and
  * we need to translate strings returned by mongoc_server_description_type() to
  * Server integer constants. */
@@ -111,16 +112,32 @@ void phongo_throw_exception(php_phongo_error_domain_t domain TSRMLS_DC, const ch
 ;
 void phongo_throw_exception_from_bson_error_t(bson_error_t *error TSRMLS_DC);
 
+/* This enum is used for processing options in phongo_execute_parse_options and
+ * selecting a libmongoc function to use in phongo_execute_command. The values
+ * are important, as READ and WRITE are also used as a bit field to determine
+ * whether readPreference, readConcern, and writeConcern options are parsed. */
+typedef enum {
+	PHONGO_OPTION_READ_CONCERN    = 0x01,
+	PHONGO_OPTION_READ_PREFERENCE = 0x02,
+	PHONGO_OPTION_WRITE_CONCERN   = 0x04,
+	PHONGO_COMMAND_RAW            = 0x07,
+	PHONGO_COMMAND_READ           = 0x03,
+	PHONGO_COMMAND_WRITE          = 0x04,
+	PHONGO_COMMAND_READ_WRITE     = 0x05,
+} php_phongo_command_type_t;
+
 zend_object_handlers *phongo_get_std_object_handlers(void);
 
-void                     phongo_server_init          (zval *return_value, mongoc_client_t *client, int server_id TSRMLS_DC);
+void                     phongo_server_init          (zval *return_value, mongoc_client_t *client, uint32_t server_id TSRMLS_DC);
+void                     phongo_session_init         (zval *return_value, mongoc_client_session_t *client_session TSRMLS_DC);
 void                     phongo_readconcern_init     (zval *return_value, const mongoc_read_concern_t *read_concern TSRMLS_DC);
 void                     phongo_readpreference_init  (zval *return_value, const mongoc_read_prefs_t *read_prefs TSRMLS_DC);
 void                     phongo_writeconcern_init    (zval *return_value, const mongoc_write_concern_t *write_concern TSRMLS_DC);
-mongoc_bulk_operation_t* phongo_bulkwrite_init       (zend_bool ordered);
-bool                     phongo_execute_write        (mongoc_client_t *client, const char *namespace, php_phongo_bulkwrite_t  *bulk_write, const mongoc_write_concern_t *write_concern, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
-int                      phongo_execute_command      (mongoc_client_t *client, const char *db, zval *zcommand, zval *zreadPreference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
-int                      phongo_execute_query        (mongoc_client_t *client, const char *namespace, zval *zquery, zval *zreadPreference, int server_id, zval *return_value, int return_value_used TSRMLS_DC);
+bool                     phongo_execute_bulk_write   (mongoc_client_t *client, const char *namespace, php_phongo_bulkwrite_t *bulk_write, zval *zwriteConcern, uint32_t server_id, zval *return_value, int return_value_used TSRMLS_DC);
+int                      phongo_execute_command      (mongoc_client_t *client, php_phongo_command_type_t type, const char *db, zval *zcommand, zval *zreadPreference, uint32_t server_id, zval *return_value, int return_value_used TSRMLS_DC);
+int                      phongo_execute_query        (mongoc_client_t *client, const char *namespace, zval *zquery, zval *zreadPreference, uint32_t server_id, zval *return_value, int return_value_used TSRMLS_DC);
+
+bool phongo_cursor_advance_and_check_for_error(mongoc_cursor_t *cursor TSRMLS_DC);
 
 const mongoc_read_concern_t*  phongo_read_concern_from_zval   (zval *zread_concern TSRMLS_DC);
 const mongoc_read_prefs_t*    phongo_read_preference_from_zval(zval *zread_preference TSRMLS_DC);
@@ -128,6 +145,11 @@ const mongoc_write_concern_t* phongo_write_concern_from_zval  (zval *zwrite_conc
 
 php_phongo_server_description_type_t php_phongo_server_description_type(mongoc_server_description_t *sd);
 
+bool phongo_parse_read_preference(zval *options, zval **zreadPreference TSRMLS_DC);
+
+zval* php_phongo_prep_legacy_option(zval *options, const char *key, bool *allocated TSRMLS_DC);
+void php_phongo_prep_legacy_option_free(zval *options TSRMLS_DC);
+
 void php_phongo_read_preference_prep_tagsets(zval *tagSets TSRMLS_DC);
 bool php_phongo_read_preference_tags_are_valid(const bson_t *tags);
 
@@ -148,6 +170,8 @@ void php_phongo_new_javascript_from_javascript_and_scope(int init, zval *object,
 void php_phongo_new_binary_from_binary_and_type(zval *object, const char *data, size_t data_len, bson_subtype_t type TSRMLS_DC);
 void php_phongo_new_decimal128(zval *object, const bson_decimal128_t *decimal TSRMLS_DC);
 void php_phongo_new_regex_from_regex_and_options(zval *object, const char *pattern, const char *flags TSRMLS_DC);
+void php_phongo_new_symbol(zval *object, const char *symbol, size_t symbol_len TSRMLS_DC);
+void php_phongo_new_dbpointer(zval *object, const char *namespace, size_t namespace_len, const bson_oid_t *oid TSRMLS_DC);
 
 zend_bool phongo_writeerror_init(zval *return_value, bson_t *bson TSRMLS_DC);
 zend_bool phongo_writeconcernerror_init(zval *return_value, bson_t *bson TSRMLS_DC);
@@ -172,7 +196,6 @@ zend_bool phongo_writeconcernerror_init(zval *return_value, bson_t *bson TSRMLS_
         ALLOC_HASHTABLE(props);                                                 \
         zend_hash_init((props), (size), NULL, ZVAL_PTR_DTOR, 0);                \
     } else if ((intern)->properties) {                                          \
-        zend_hash_clean((intern)->properties);                                  \
         (props) = (intern)->properties;                                         \
     } else {                                                                    \
         ALLOC_HASHTABLE(props);                                                 \
@@ -181,6 +204,9 @@ zend_bool phongo_writeconcernerror_init(zval *return_value, bson_t *bson TSRMLS_
     }                                                                           \
 } while(0);
 
+#define PHONGO_ZVAL_CLASS_OR_TYPE_NAME(zv) (Z_TYPE(zv) == IS_OBJECT ? ZSTR_VAL(Z_OBJCE(zv)->name) : zend_get_type_by_const(Z_TYPE(zv)))
+#define PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(zvp) PHONGO_ZVAL_CLASS_OR_TYPE_NAME(*(zvp))
+
 #endif /* PHONGO_H */
 
 
