@@ -15,40 +15,26 @@
  */
 
 #ifdef HAVE_CONFIG_H
-#	include "config.h"
+# include "config.h"
 #endif
 
-/* External libs */
-#include <bson.h>
-#include <mongoc.h>
-
-/* PHP Core stuff */
 #include <php.h>
-#include <php_ini.h>
-#include <ext/standard/info.h>
 #include <Zend/zend_interfaces.h>
-#include <ext/spl/spl_iterators.h>
 #include <ext/standard/php_var.h>
 #if PHP_VERSION_ID >= 70000
 # include <zend_smart_str.h>
 #else
 # include <ext/standard/php_smart_str.h>
 #endif
-/* Our Compatability header */
+
 #include "phongo_compat.h"
-
-/* Our stuffz */
 #include "php_phongo.h"
-#include "php_bson.h"
 
-
-PHONGO_API zend_class_entry *php_phongo_timestamp_ce;
-
-zend_object_handlers php_phongo_handler_timestamp;
+zend_class_entry *php_phongo_timestamp_ce;
 
 /* Initialize the object and return whether it was successful. An exception will
  * be thrown on error. */
-static bool php_phongo_timestamp_init(php_phongo_timestamp_t *intern, int64_t increment, int64_t timestamp TSRMLS_DC)
+static bool php_phongo_timestamp_init(php_phongo_timestamp_t *intern, int64_t increment, int64_t timestamp TSRMLS_DC) /* {{{ */
 {
 	if (increment < 0 || increment > UINT32_MAX) {
 		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected increment to be an unsigned 32-bit integer, %" PHONGO_LONG_FORMAT " given", increment);
@@ -65,11 +51,11 @@ static bool php_phongo_timestamp_init(php_phongo_timestamp_t *intern, int64_t in
 	intern->initialized = true;
 
 	return true;
-}
+} /* }}} */
 
 /* Initialize the object from numeric strings and return whether it was
  * successful. An exception will be thrown on error. */
-static bool php_phongo_timestamp_init_from_string(php_phongo_timestamp_t *intern, const char *s_increment, phongo_zpp_char_len s_increment_len, const char *s_timestamp, phongo_zpp_char_len s_timestamp_len TSRMLS_DC)
+static bool php_phongo_timestamp_init_from_string(php_phongo_timestamp_t *intern, const char *s_increment, phongo_zpp_char_len s_increment_len, const char *s_timestamp, phongo_zpp_char_len s_timestamp_len TSRMLS_DC) /* {{{ */
 {
 	int64_t increment, timestamp;
 	char *endptr = NULL;
@@ -99,11 +85,11 @@ static bool php_phongo_timestamp_init_from_string(php_phongo_timestamp_t *intern
 	}
 
 	return php_phongo_timestamp_init(intern, increment, timestamp TSRMLS_CC);
-}
+} /* }}} */
 
 /* Initialize the object from a HashTable and return whether it was successful.
  * An exception will be thrown on error. */
-static bool php_phongo_timestamp_init_from_hash(php_phongo_timestamp_t *intern, HashTable *props TSRMLS_DC)
+static bool php_phongo_timestamp_init_from_hash(php_phongo_timestamp_t *intern, HashTable *props TSRMLS_DC) /* {{{ */
 {
 #if PHP_VERSION_ID >= 70000
 	zval *increment, *timestamp;
@@ -131,12 +117,12 @@ static bool php_phongo_timestamp_init_from_hash(php_phongo_timestamp_t *intern,
 
 	phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "%s initialization requires \"increment\" and \"timestamp\" integer or numeric string fields", ZSTR_VAL(php_phongo_timestamp_ce->name));
 	return false;
-}
+} /* }}} */
 
-/* {{{ proto void Timestamp::__construct(int|string $increment, int|string $timestamp)
+/* {{{ proto void MongoDB\BSON\Timestamp::__construct(int|string $increment, int|string $timestamp)
    Construct a new BSON timestamp type, which consists of a 4-byte increment and
    4-byte timestamp. */
-PHP_METHOD(Timestamp, __construct)
+static PHP_METHOD(Timestamp, __construct)
 {
 	php_phongo_timestamp_t *intern;
 	zend_error_handling     error_handling;
@@ -161,7 +147,7 @@ PHP_METHOD(Timestamp, __construct)
 	}
 
 	if (Z_TYPE_P(increment) != IS_STRING) {
-		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected increment to be an unsigned 32-bit integer or string, %s given", zend_get_type_by_const(Z_TYPE_P(increment)));
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected increment to be an unsigned 32-bit integer or string, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(increment));
 		return;
 	}
 
@@ -170,17 +156,46 @@ PHP_METHOD(Timestamp, __construct)
 	}
 
 	if (Z_TYPE_P(timestamp) != IS_STRING) {
-		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected timestamp to be an unsigned 32-bit integer or string, %s given", zend_get_type_by_const(Z_TYPE_P(timestamp)));
+		phongo_throw_exception(PHONGO_ERROR_INVALID_ARGUMENT TSRMLS_CC, "Expected timestamp to be an unsigned 32-bit integer or string, %s given", PHONGO_ZVAL_CLASS_OR_TYPE_NAME_P(timestamp));
 		return;
 	}
 
 	php_phongo_timestamp_init_from_string(intern, Z_STRVAL_P(increment), Z_STRLEN_P(increment), Z_STRVAL_P(timestamp), Z_STRLEN_P(timestamp) TSRMLS_CC);
-}
-/* }}} */
+} /* }}} */
 
-/* {{{ proto void Timestamp::__set_state(array $properties)
+/* {{{ proto integer MongoDB\BSON\Timestamp::getIncrement()
 */
-PHP_METHOD(Timestamp, __set_state)
+static PHP_METHOD(Timestamp, getIncrement)
+{
+	php_phongo_timestamp_t *intern;
+
+	intern = Z_TIMESTAMP_OBJ_P(getThis());
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	RETVAL_LONG(intern->increment);
+} /* }}} */
+
+/* {{{ proto integer MongoDB\BSON\Timestamp::getTimestamp()
+*/
+static PHP_METHOD(Timestamp, getTimestamp)
+{
+	php_phongo_timestamp_t *intern;
+
+	intern = Z_TIMESTAMP_OBJ_P(getThis());
+
+	if (zend_parse_parameters_none() == FAILURE) {
+		return;
+	}
+
+	RETVAL_LONG(intern->timestamp);
+} /* }}} */
+
+/* {{{ proto void MongoDB\BSON\Timestamp::__set_state(array $properties)
+*/
+static PHP_METHOD(Timestamp, __set_state)
 {
 	php_phongo_timestamp_t *intern;
 	HashTable              *props;
@@ -196,12 +211,11 @@ PHP_METHOD(Timestamp, __set_state)
 	props = Z_ARRVAL_P(array);
 
 	php_phongo_timestamp_init_from_hash(intern, props TSRMLS_CC);
-}
-/* }}} */
+} /* }}} */
 
-/* {{{ proto string Timestamp::__toString()
+/* {{{ proto string MongoDB\BSON\Timestamp::__toString()
    Returns a string in the form: [increment:timestamp] */
-PHP_METHOD(Timestamp, __toString)
+static PHP_METHOD(Timestamp, __toString)
 {
 	php_phongo_timestamp_t    *intern;
 	char                      *retval;
@@ -217,12 +231,11 @@ PHP_METHOD(Timestamp, __toString)
 	retval_len = spprintf(&retval, 0, "[%" PRIu32 ":%" PRIu32 "]", intern->increment, intern->timestamp);
 	PHONGO_RETVAL_STRINGL(retval, retval_len);
 	efree(retval);
-}
-/* }}} */
+} /* }}} */
 
-/* {{{ proto array Timestamp::jsonSerialize()
+/* {{{ proto array MongoDB\BSON\Timestamp::jsonSerialize()
 */
-PHP_METHOD(Timestamp, jsonSerialize)
+static PHP_METHOD(Timestamp, jsonSerialize)
 {
 	php_phongo_timestamp_t *intern;
 
@@ -254,12 +267,11 @@ PHP_METHOD(Timestamp, jsonSerialize)
 		ADD_ASSOC_ZVAL_EX(return_value, "$timestamp", ts);
 	}
 #endif
-}
-/* }}} */
+} /* }}} */
 
-/* {{{ proto string Timestamp::serialize()
+/* {{{ proto string MongoDB\BSON\Timestamp::serialize()
 */
-PHP_METHOD(Timestamp, serialize)
+static PHP_METHOD(Timestamp, serialize)
 {
 	php_phongo_timestamp_t   *intern;
 #if PHP_VERSION_ID >= 70000
@@ -303,12 +315,11 @@ PHP_METHOD(Timestamp, serialize)
 
 	smart_str_free(&buf);
 	zval_ptr_dtor(&retval);
-}
-/* }}} */
+} /* }}} */
 
-/* {{{ proto string Timestamp::unserialize(string $serialized)
+/* {{{ proto void MongoDB\BSON\Timestamp::unserialize(string $serialized)
 */
-PHP_METHOD(Timestamp, unserialize)
+static PHP_METHOD(Timestamp, unserialize)
 {
 	php_phongo_timestamp_t *intern;
 	zend_error_handling     error_handling;
@@ -350,11 +361,9 @@ PHP_METHOD(Timestamp, unserialize)
 	php_phongo_timestamp_init_from_hash(intern, HASH_OF(props) TSRMLS_CC);
 #endif
 	zval_ptr_dtor(&props);
-}
-/* }}} */
-
-/* {{{ BSON\Timestamp */
+} /* }}} */
 
+/* {{{ MongoDB\BSON\Timestamp function entries */
 ZEND_BEGIN_ARG_INFO_EX(ai_Timestamp___construct, 0, 0, 2)
 	ZEND_ARG_INFO(0, increment)
 	ZEND_ARG_INFO(0, timestamp)
@@ -378,25 +387,32 @@ static zend_function_entry php_phongo_timestamp_me[] = {
 	PHP_ME(Timestamp, jsonSerialize, ai_Timestamp_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_ME(Timestamp, serialize, ai_Timestamp_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_ME(Timestamp, unserialize, ai_Timestamp_unserialize, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Timestamp, getIncrement, ai_Timestamp_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
+	PHP_ME(Timestamp, getTimestamp, ai_Timestamp_void, ZEND_ACC_PUBLIC|ZEND_ACC_FINAL)
 	PHP_FE_END
 };
-
 /* }}} */
 
+/* {{{ MongoDB\BSON\Timestamp object handlers */
+static zend_object_handlers php_phongo_handler_timestamp;
 
-/* {{{ php_phongo_timestamp_t object handlers */
 static void php_phongo_timestamp_free_object(phongo_free_object_arg *object TSRMLS_DC) /* {{{ */
 {
 	php_phongo_timestamp_t *intern = Z_OBJ_TIMESTAMP(object);
 
 	zend_object_std_dtor(&intern->std TSRMLS_CC);
 
+	if (intern->properties) {
+		zend_hash_destroy(intern->properties);
+		FREE_HASHTABLE(intern->properties);
+	}
+
 #if PHP_VERSION_ID < 70000
 	efree(intern);
 #endif
 } /* }}} */
 
-phongo_create_object_retval php_phongo_timestamp_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
+static phongo_create_object_retval php_phongo_timestamp_create_object(zend_class_entry *class_type TSRMLS_DC) /* {{{ */
 {
 	php_phongo_timestamp_t *intern = NULL;
 
@@ -444,10 +460,10 @@ static HashTable *php_phongo_timestamp_get_gc(zval *object, phongo_get_gc_table
 	*table = NULL;
 	*n = 0;
 
-	return zend_std_get_properties(object TSRMLS_CC);
+	return Z_TIMESTAMP_OBJ_P(object)->properties;
 } /* }}} */
 
-HashTable *php_phongo_timestamp_get_properties(zval *object TSRMLS_DC) /* {{{ */
+static HashTable *php_phongo_timestamp_get_properties_hash(zval *object, bool is_debug TSRMLS_DC) /* {{{ */
 {
 	php_phongo_timestamp_t *intern;
 	HashTable              *props;
@@ -457,7 +473,8 @@ HashTable *php_phongo_timestamp_get_properties(zval *object TSRMLS_DC) /* {{{ */
 	int                     s_timestamp_len;
 
 	intern = Z_TIMESTAMP_OBJ_P(object);
-	props = zend_std_get_properties(object TSRMLS_CC);
+
+	PHONGO_GET_PROPERTY_HASH_INIT_PROPS(is_debug, intern, props, 2);
 
 	if (!intern->initialized) {
 		return props;
@@ -492,37 +509,43 @@ HashTable *php_phongo_timestamp_get_properties(zval *object TSRMLS_DC) /* {{{ */
 
 	return props;
 } /* }}} */
+
+static HashTable *php_phongo_timestamp_get_debug_info(zval *object, int *is_temp TSRMLS_DC) /* {{{ */
+{
+	*is_temp = 1;
+	return php_phongo_timestamp_get_properties_hash(object, true TSRMLS_CC);
+} /* }}} */
+
+static HashTable *php_phongo_timestamp_get_properties(zval *object TSRMLS_DC) /* {{{ */
+{
+	return php_phongo_timestamp_get_properties_hash(object, false TSRMLS_CC);
+} /* }}} */
 /* }}} */
 
-/* {{{ PHP_MINIT_FUNCTION */
-PHP_MINIT_FUNCTION(Timestamp)
+void php_phongo_timestamp_init_ce(INIT_FUNC_ARGS) /* {{{ */
 {
 	zend_class_entry ce;
-	(void)type;(void)module_number;
 
 	INIT_NS_CLASS_ENTRY(ce, "MongoDB\\BSON", "Timestamp", php_phongo_timestamp_me);
 	php_phongo_timestamp_ce = zend_register_internal_class(&ce TSRMLS_CC);
 	php_phongo_timestamp_ce->create_object = php_phongo_timestamp_create_object;
 	PHONGO_CE_FINAL(php_phongo_timestamp_ce);
 
+	zend_class_implements(php_phongo_timestamp_ce TSRMLS_CC, 1, php_phongo_timestamp_interface_ce);
 	zend_class_implements(php_phongo_timestamp_ce TSRMLS_CC, 1, php_phongo_json_serializable_ce);
 	zend_class_implements(php_phongo_timestamp_ce TSRMLS_CC, 1, php_phongo_type_ce);
 	zend_class_implements(php_phongo_timestamp_ce TSRMLS_CC, 1, zend_ce_serializable);
 
 	memcpy(&php_phongo_handler_timestamp, phongo_get_std_object_handlers(), sizeof(zend_object_handlers));
 	php_phongo_handler_timestamp.compare_objects = php_phongo_timestamp_compare_objects;
+	php_phongo_handler_timestamp.get_debug_info = php_phongo_timestamp_get_debug_info;
 	php_phongo_handler_timestamp.get_gc = php_phongo_timestamp_get_gc;
 	php_phongo_handler_timestamp.get_properties = php_phongo_timestamp_get_properties;
 #if PHP_VERSION_ID >= 70000
 	php_phongo_handler_timestamp.free_obj = php_phongo_timestamp_free_object;
 	php_phongo_handler_timestamp.offset = XtOffsetOf(php_phongo_timestamp_t, std);
 #endif
-
-	return SUCCESS;
-}
-/* }}} */
-
-
+} /* }}} */
 
 /*
  * Local variables:
